//! This module consists of endpoints to handle callbacks

mod delete_message;
mod get;
mod get_menu;

pub use delete_message::delete_message;
pub use get::get;
pub use get_menu::get_menu;

use crate::errors::InvalidCommand;
use base64::{engine::general_purpose::STANDARD_NO_PAD as B64_ENGINE, Engine as _};
use std::str::FromStr;
use teloxide::types::CallbackQuery;

type NameHash = Vec<u8>;

#[derive(Clone, Copy, Debug)]
pub enum AlterableField {
    Name,
    Login,
    Pass,
}

#[derive(Clone, Debug)]
pub enum CallbackCommand {
    DeleteMessage,
    Get(NameHash),
    GetMenu,
    Decrypt(NameHash),
    Hide(NameHash),
    Alter(NameHash, AlterableField),
    DeleteAccount(NameHash),
}

impl CallbackCommand {
    pub fn from_query(q: CallbackQuery) -> Option<Self> {
        q.message.as_ref()?;
        q.data.and_then(|text| dbg!(text.parse()).ok())
    }
}

impl FromStr for CallbackCommand {
    type Err = crate::errors::InvalidCommand;

    fn from_str(s: &str) -> Result<Self, Self::Err> {
        use AlterableField::*;
        use CallbackCommand::*;

        println!("{s}");

        match s {
            "delete_message" => return Ok(DeleteMessage),
            "get_menu" => return Ok(GetMenu),
            _ => (),
        };

        let mut substrings = s.split(' ');
        let (command, name) = match (substrings.next(), substrings.next(), substrings.next()) {
            (Some(command), Some(name), None) => (command, name),
            _ => return Err(InvalidCommand::InvalidParams),
        };
        let name_hash = B64_ENGINE.decode(name)?;
        if name_hash.len() != 32 {
            return Err(InvalidCommand::InvalidOutputLength);
        }

        match command {
            "get" => Ok(Get(name_hash)),
            "decrypt" => Ok(Decrypt(name_hash)),
            "hide" => Ok(Hide(name_hash)),
            "an" => Ok(Alter(name_hash, Name)),
            "al" => Ok(Alter(name_hash, Login)),
            "ap" => Ok(Alter(name_hash, Pass)),
            "delete" => Ok(DeleteAccount(name_hash)),
            _ => Err(InvalidCommand::InvalidParams),
        }
    }
}
