use crate::{change_state, prelude::*};
use tokio::task::spawn_blocking;

/// Actually sets the master password
async fn get_master_pass(
    bot: Throttle<Bot>,
    msg: Message,
    db: DatabaseConnection,
    dialogue: MainDialogue,
    ids: MessageIds,
    master_pass: String,
) -> crate::Result<()> {
    ids.delete(&bot).await;
    dialogue.exit().await?;

    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;
    let model =
        spawn_blocking(move || master_pass::ActiveModel::from_unencrypted(user_id, &master_pass))
            .await?;
    model.insert(&db).await?;
    bot.send_message(msg.chat.id, "Success")
        .reply_markup(deletion_markup())
        .await?;
    Ok(())
}

/// Handles /set_master_pass command
pub async fn set_master_pass(
    bot: Throttle<Bot>,
    msg: Message,
    dialogue: MainDialogue,
    db: DatabaseConnection,
) -> crate::Result<()> {
    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;
    if MasterPass::exists(user_id, &db).await? {
        bot.send_message(msg.chat.id, "Master password already exists")
            .reply_markup(deletion_markup())
            .await?;
        return Ok(());
    }
    let previous = bot
        .send_message(msg.chat.id, "Send new master password")
        .await?;

    change_state!(
        dialogue,
        previous,
        (),
        State::GetNewMasterPass,
        get_master_pass
    )
}
